'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initialReducerState = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /* eslint-disable no-underscore-dangle */
// WP REST API has many underscore dangle's, no point to fight against it


var _defaultsDeep = require('lodash/defaultsDeep');

var _defaultsDeep2 = _interopRequireDefault(_defaultsDeep);

var _nthArg = require('lodash/nthArg');

var _nthArg2 = _interopRequireDefault(_nthArg);

var _mapKeys = require('lodash/mapKeys');

var _mapKeys2 = _interopRequireDefault(_mapKeys);

var _forEach = require('lodash/forEach');

var _forEach2 = _interopRequireDefault(_forEach);

var _find = require('lodash/find');

var _find2 = _interopRequireDefault(_find);

var _noop = require('lodash/noop');

var _noop2 = _interopRequireDefault(_noop);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _selectors = require('./selectors');

var _wpapi = require('./adapters/wpapi');

var _wpapi2 = _interopRequireDefault(_wpapi);

var _symbols = require('./symbols');

var _actions = require('./constants/actions');

var _requestStatus = require('./constants/requestStatus');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var initialReducerState = exports.initialReducerState = _immutable2.default.fromJS({
  requestsByName: {},
  requestsByQuery: {},

  resources: [],
  resourcesIndexes: {}
});

var ReduxWPAPI = function () {
  function ReduxWPAPI(_ref) {
    var _this = this;

    var adapter = _ref.adapter;

    var settings = _objectWithoutProperties(_ref, ['adapter']);

    _classCallCheck(this, ReduxWPAPI);

    this.middleware = function (store) {
      return function (next) {
        return function (action) {
          if (!action || action.type !== _actions.REDUX_WP_API_CALL) return next(action);

          var request = _this.adapter.buildRequest(action.payload);
          var meta = {
            name: action.payload.name,
            aggregator: _this.adapter.getAggregator(_this.adapter.getUrl(request)),
            operation: _this.adapter.getOperation(request),
            requestAt: Date.now()
          };

          var payload = {};
          if (meta.operation === 'get') {
            var cache = void 0;
            var lastCacheUpdate = void 0;
            var data = void 0;
            var ttl = void 0;
            if (_this.adapter.getTTL) {
              ttl = _this.adapter.getTTL(request);
            }

            if (ttl !== 0 && !ttl) {
              ttl = _this.settings.ttl;
            }

            var state = store.getState().wp;
            var indexes = _this.adapter.getIndexes(request);
            var resourceLocalID = _this.getResourceLocalID(state, meta.aggregator, indexes);

            payload.cacheID = _this.adapter.generateCacheID(request);
            payload.page = parseInt(_this.adapter.getRequestedPage(request) || 1, 10);

            if (resourceLocalID !== false) {
              cache = state.getIn(['resources', resourceLocalID]);
              data = [resourceLocalID];
            }

            if (cache) {
              lastCacheUpdate = cache[_symbols.lastCacheUpdate];
              if (cache[_symbols.partial]) {
                ttl = 0;
              }
            } else {
              cache = state.getIn(['requestsByQuery', payload.cacheID, payload.page]);
              data = state.get('data');

              if (cache && cache.get('error')) {
                cache = false;
              }
            }

            if (cache) {
              lastCacheUpdate = lastCacheUpdate || cache.get('responseAt') || cache.get('requestAt');
              next({
                meta: meta,
                type: _actions.REDUX_WP_API_CACHE_HIT,
                payload: {
                  cacheID: payload.cacheID,
                  page: payload.page,
                  lastCacheUpdate: lastCacheUpdate,
                  data: data
                }
              });

              if (Date.now() - lastCacheUpdate < ttl) {
                return Promise.resolve((0, _selectors.selectRequest)({
                  cacheID: payload.cacheID,
                  page: payload.page
                })(store.getState()));
              }
            }
          }

          next({
            type: _actions.REDUX_WP_API_REQUEST,
            payload: payload,
            meta: meta
          });

          return _this.adapter.sendRequest(request).then(function (response) {
            return next({
              type: _actions.REDUX_WP_API_SUCCESS,
              payload: _extends({}, payload, { response: response }),
              meta: _extends({}, meta, { responseAt: Date.now() })
            });
          }, function (error) {
            return next({
              type: _actions.REDUX_WP_API_FAILURE,
              payload: payload,
              error: error,
              meta: _extends({}, meta, { responseAt: Date.now() })
            });
          }).then(function () {
            if (meta.operation === 'get') {
              return (0, _selectors.selectRequest)({
                cacheID: payload.cacheID,
                page: payload.page
              })(store.getState());
            }

            return (0, _selectors.selectRequest)(meta.name)(store.getState());
          });
        };
      };
    };

    this.reducer = function () {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialReducerState;
      var action = arguments[1];

      switch (action.type) {
        case _actions.REDUX_WP_API_CACHE_HIT:
          {
            var _action$payload = action.payload;
            var data = _action$payload.data;
            var page = _action$payload.page;
            var cacheID = _action$payload.cacheID;

            var newState = state.mergeIn(['requestsByName', action.meta.name], { page: page, cacheID: cacheID });

            if (!newState.getIn(['requestsByQuery', cacheID, page])) {
              newState = newState.mergeIn(['requestsByQuery', cacheID, page], {
                status: _requestStatus.resolved,
                operation: action.meta.operation,
                error: false,
                requestAt: action.payload.lastCacheUpdate,
                responseAt: action.payload.lastCacheUpdate
              }).setIn(['requestsByQuery', cacheID, 1, 'data'], data);
            }

            return newState;
          }

        case _actions.REDUX_WP_API_REQUEST:
          {
            var _action$meta = action.meta;
            var name = _action$meta.name;
            var operation = _action$meta.operation;
            var requestAt = _action$meta.requestAt;


            var requestState = {
              status: _requestStatus.pending,
              requestAt: requestAt,
              operation: operation
            };

            if (operation === 'get') {
              var _action$payload2 = action.payload;
              var _page = _action$payload2.page;
              var _cacheID = _action$payload2.cacheID;

              var pagination = state.getIn(['requestsByQuery', _cacheID, 'pagination']);
              pagination = pagination || {};

              return state.mergeIn(['requestsByName', name], { page: _page, cacheID: _cacheID }).setIn(['requestsByQuery', _cacheID, 'pagination'], pagination).mergeIn(['requestsByQuery', _cacheID, _page], requestState);
            }

            return state.mergeIn(['requestsByName', name], requestState).setIn(['requestsByName', name, 'data'], false);
          }
        case _actions.REDUX_WP_API_SUCCESS:
          {
            var _ret = function () {
              var _additionalData;

              var newState = state;
              var payload = action.payload;
              var _action$meta2 = action.meta;
              var name = _action$meta2.name;
              var aggregator = _action$meta2.aggregator;
              var requestAt = _action$meta2.requestAt;
              var responseAt = _action$meta2.responseAt;
              var cacheID = payload.cacheID;
              var page = payload.page;
              var response = payload.response;

              var body = response;

              var requestState = {
                responseAt: responseAt,
                status: _requestStatus.resolved,
                error: false
              };

              if (!(0, _isArray2.default)(body)) {
                body = [body];
              }

              var data = [];
              var additionalData = (_additionalData = {}, _defineProperty(_additionalData, _symbols.lastCacheUpdate, requestState.responseAt), _defineProperty(_additionalData, _symbols.partial, false), _additionalData);

              body.forEach(function (resource) {
                newState = _this.indexResource(newState, aggregator, resource, additionalData, function (id) {
                  data.push(id);
                });
              });

              if (action.meta.operation === 'get') {
                var _pagination = _this.adapter.getPagination(response);
                newState = newState.mergeIn(['requestsByQuery', cacheID, page], requestState);
                newState = newState.setIn(['requestsByQuery', cacheID, page, 'data'], data);
                newState = newState.setIn(['requestsByQuery', cacheID, 'pagination'], _pagination);
              } else if (newState.getIn(['requestsByName', name, 'requestAt']) === requestAt) {
                newState = newState.setIn(['requestsByName', action.meta.name, 'data'], data).mergeIn(['requestsByName', name], requestState);
              }

              return {
                v: newState
              };
            }();

            if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
          }
        case _actions.REDUX_WP_API_FAILURE:
          {
            var error = action.error;
            var _action$payload3 = action.payload;
            var _page2 = _action$payload3.page;
            var _cacheID2 = _action$payload3.cacheID;

            var _requestState = {
              status: _requestStatus.rejected,
              error: {
                message: error.message,
                status: error.status
              }
            };

            if (action.meta.operation === 'get') {
              return state.mergeIn(['requestsByQuery', _cacheID2, _page2], _requestState);
            }

            return state.mergeIn(['requestsByName', action.meta.name], _requestState);
          }
        default:
          return state;
      }
    };

    this.settings = (0, _defaultsDeep2.default)({}, settings, ReduxWPAPI.defaultSettings);

    if (!adapter) {
      this.adapter = new _wpapi2.default(this.settings);
    } else {
      this.adapter = adapter;
    }

    this.settings.customCacheIndexes = _extends({}, this.adapter.customCacheIndexes || {}, this.settings.customCacheIndexes);
  }

  _createClass(ReduxWPAPI, [{
    key: 'getResourceLocalID',
    value: function getResourceLocalID(state, aggregator, resource) {
      var indexers = this.settings.customCacheIndexes[aggregator];
      var indexBy = (0, _find2.default)((0, _isArray2.default)(indexers) ? ['id'].concat(indexers) : ['id', indexers], function (indexer) {
        return !(0, _isUndefined2.default)(resource[indexer]) && !(0, _isUndefined2.default)(state.getIn(['resourcesIndexes', aggregator, indexer, resource[indexer].toString()]));
      });

      if ((0, _isUndefined2.default)(indexBy)) {
        return false;
      }

      return state.getIn(['resourcesIndexes', aggregator, indexBy, resource[indexBy].toString()]);
    }
  }, {
    key: 'indexResource',
    value: function indexResource(state, aggregator, resource, meta) {
      var _this2 = this;

      var onIndex = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : _noop2.default;

      var newState = state;
      var _embedded = void 0;
      var curies = (resource._links || {}).curies;
      var _links = this.resolveAliases(resource._links, curies) || {};
      delete _links.curies;

      var resourceLocalID = this.getResourceLocalID(state, aggregator, resource);
      var emptyState = {};
      var oldState = emptyState;
      if (resourceLocalID !== false) {
        oldState = newState.getIn(['resources', resourceLocalID]);
      }

      if (resource._embedded) {
        (function () {
          _embedded = _extends({}, (oldState || {})._embedded);
          var alisedEmbedded = _this2.resolveAliases(resource._embedded, curies);
          var embeddedMeta = _extends({}, meta, _defineProperty({}, _symbols.partial, true));
          (0, _forEach2.default)(alisedEmbedded, function (embeddable, relName) {
            (0, _forEach2.default)(_links[relName], function (link, index) {
              var embeddedAggregator = _this2.adapter.getAggregator(link.href);
              if (!embeddedAggregator) return;

              var toEmbed = alisedEmbedded[relName][index];
              toEmbed = ((0, _isArray2.default)(toEmbed) ? toEmbed : [toEmbed]).reduce(function (collection, toBeEmbedded) {
                if (toBeEmbedded.code === 'rest_no_route') return collection;

                newState = _this2.indexResource(newState, embeddedAggregator, toBeEmbedded, embeddedMeta, function (id) {
                  return collection.push(id);
                });

                return collection;
              }, []);

              if (!(0, _isArray2.default)(alisedEmbedded[relName][index])) {
                if (!toEmbed.length) {
                  return;
                }

                toEmbed = toEmbed[0];
              }

              _embedded[_this2.adapter.embedLinkAs(_extends({ name: relName }, link))] = toEmbed;
            });
          });
        })();
      }

      if (resourceLocalID === false) {
        resourceLocalID = newState.get('resources').size;
      }

      var isPartial = meta[_symbols.partial] || false;
      if (oldState !== emptyState && oldState[_symbols.partial] && !meta[_symbols.partial]) {
        isPartial = false;
      }

      var resourceTransformed = _extends({}, oldState, resource, meta, _defineProperty({
        _links: _links,
        _embedded: _embedded
      }, _symbols.partial, isPartial));

      if (this.adapter.transformResource) {
        resourceTransformed = this.adapter.transformResource(resourceTransformed);
      }

      if (this.settings.transformResource) {
        resourceTransformed = this.settings.transformResource(resourceTransformed);
      }

      newState = newState.setIn(['resources', resourceLocalID], resourceTransformed);
      var indexers = this.settings.customCacheIndexes[aggregator];

      (0, _forEach2.default)((0, _isArray2.default)(indexers) ? ['id'].concat(indexers) : ['id', indexers], function (indexer) {
        if (!(0, _isUndefined2.default)(resource[indexer])) {
          newState = newState.setIn(['resourcesIndexes', aggregator, indexer, resource[indexer].toString()], resourceLocalID);
        }
      });

      onIndex(resourceLocalID);
      return newState;
    }
  }, {
    key: 'resolveAliases',
    value: function resolveAliases(map, curies) {
      return (curies || []).reduce(function (remapped, cury) {
        var alias = new RegExp('^' + cury.name + ':');
        return (0, _mapKeys2.default)(remapped, function (value, key) {
          return cury.href.replace(/\{rel\}/g, key.replace(alias, ''));
        });
      }, map) || map;
    }
  }]);

  return ReduxWPAPI;
}();

ReduxWPAPI.displayName = '[REDUX-WP-API]';
ReduxWPAPI.defaultSettings = {
  transformResource: (0, _nthArg2.default)(0),
  timeout: 30000,
  ttl: 60000
};
exports.default = ReduxWPAPI;