'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _find = require('lodash/find');

var _find2 = _interopRequireDefault(_find);

var _findKey = require('lodash/findKey');

var _findKey2 = _interopRequireDefault(_findKey);

var _reduce = require('lodash/reduce');

var _reduce2 = _interopRequireDefault(_reduce);

var _capitalize = require('lodash/capitalize');

var _capitalize2 = _interopRequireDefault(_capitalize);

var _camelCase = require('lodash/camelCase');

var _camelCase2 = _interopRequireDefault(_camelCase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * This adapter connects `node-wpapi` to `redux-wpapi`, abstracting any
 * specificity.
 *
 * @type {adapter}
 */
var WPAPIAdapter = function () {

  /**
   * Sets up the adapter with the API settings
   *
   * @param {Object} settings
   */

  /**
  * Regular Expression to identify a capture group in PCRE formats
  * `(?<name>regex)`, `(?'name'regex)` or `(?P<name>regex)` (see
  * regular-expressions.info/refext.html); RegExp is built as a string
  * to enable more detailed annotation.
  *
  * @type {RegExp}
  * @author kadamwhite
  * @link https://github.com/WP-API/node-wpapi/blob/0b4fbe8740fe4bf5b8c4e1a5c06127a9325d5d6d/lib/util/named-group-regexp.js
  */
  function WPAPIAdapter(settings) {
    _classCallCheck(this, WPAPIAdapter);

    this.customCacheIndexes = {
      taxonomies: ['slug']
    };

    if (!settings.api) {
      throw new Error('[ReduxWPAPI WPAPI Adapter]: api client must be provided');
    }

    this.defaultTTL = settings.ttl;
    this.api = settings.api;
  }

  /**
   * Transforms the resource before indexing
   *
   * This is useful to remove unused attributes.
   *
   * @param  {Object} resource Each individual resource
   * @return {Object}          Resource transformed
   */


  /**
   * Aggregators' unique properties to be indexed
   */


  _createClass(WPAPIAdapter, [{
    key: 'transformResource',
    value: function transformResource(resource) {
      return resource;
    }

    /**
     * Extracts current page from a given request
     *
     * @param {Object} request Request provided by consumer through action `request` param
     * @return {Number} current page
     */

  }, {
    key: 'getRequestedPage',
    value: function getRequestedPage(_ref) {
      var wpRequest = _ref.wpRequest;

      return wpRequest._params.page;
    }

    /**
     * Extracts pagination params from Response
     *
     * It is up to the api decide the shape of pagination, `totalPages` and `total` are a common case.
     *
     * @param {Object|Array} response Response resulted by `callAPI` operation
     * @return {Object}               Pagination params such as `total` and `totalPages`
     */

  }, {
    key: 'getPagination',
    value: function getPagination(response) {
      var _ref2 = response._paging || {};

      var total = _ref2.total;
      var totalPages = _ref2.totalPages;

      return {
        total: parseInt(total || 1, 10),
        totalPages: parseInt(totalPages || 1, 10)
      };
    }

    /**
     * Extracts params from request for finding local cache.
     *
     * These params are going to be used to search resource within local cache. It will work only with
     * previously indexed in that aggregator.
     *
     * FIXME: provisional method while pending WP-API/node-wpapi#213
     *
     * @param {Object} request Request provided by consumer through action `request` param
     * @return {Object} params used to search resource within local cache.
     */

  }, {
    key: 'getIndexes',
    value: function getIndexes(_ref3) {
      var wpRequest = _ref3.wpRequest;

      /* eslint-disable no-param-reassign */
      // Internal mutations inside reduce function
      var lastFragment = null;
      var unresolvedNesting = 1;
      var foundIndexers = (0, _reduce2.default)(wpRequest._path, function (indexers, fragment, piece) {
        var id = (0, _find2.default)(wpRequest._levels[piece], function (cmp) {
          return cmp.validate(fragment);
        });

        if (!id) return indexers;
        var name = id.component.match(WPAPIAdapter.namedGroupRegex);

        if (name) {
          lastFragment = indexers[name[1]] = fragment;
          unresolvedNesting--;
        } else {
          unresolvedNesting++;
        }

        return indexers;
      }, _extends({}, wpRequest._params));

      if (!foundIndexers.id && lastFragment && unresolvedNesting === 1) {
        foundIndexers.id = lastFragment;
      }

      return foundIndexers;
    }

    /**
     * Generates Cache ID
     *
     * This Cache ID should be the same for all requests that have the same query and differs only by
     * pagination arguments.
     *
     * @param {Object} request Request provided by consumer through action `request` param
     * @return {String}        A Cache ID to given request
     */

  }, {
    key: 'generateCacheID',
    value: function generateCacheID(_ref4) {
      var wpRequest = _ref4.wpRequest;

      /* eslint-disable no-param-reassign */
      // Internal mutations are reversed before return
      var page = wpRequest._params.page;
      var _embed = wpRequest._params._embed;

      delete wpRequest._params.page;
      delete wpRequest._params._embed;

      var cacheID = wpRequest._renderURI();

      wpRequest._params.page = page;
      wpRequest._params._embed = _embed;

      return cacheID.replace(this.api._options.endpoint, '');
    }

    /**
     * Give name to a given link (relationship)
     *
     * Decides how embedded resources brought by a given link will be identified after denormalization
     *
     * @param {Object} link      Relationship object between resources
     * @param {String} link.name Relationship name between resources
     * @param {String} link.href Source of current link's resources
     * @return {String}          Property to which resources will be embedded after denormalization
     */

  }, {
    key: 'embedLinkAs',
    value: function embedLinkAs(link) {
      var linkRenamed = link.name.replace(/^https:\/\/api\.w\.org\//, '');

      switch (linkRenamed) {
        case 'featuredmedia':
          linkRenamed = 'featured_media';break;
        case 'term':
          linkRenamed = link.href.replace(/\?.*$/, '').replace(/.*\/([^\/]+)$/, '$1');
          break;
        default:
      }

      return linkRenamed;
    }

    /**
     * Converts a request into a URL
     *
     * This URL is needed in order to get its resources aggregator.
     *
     * @param  {Object} request Request provided by consumer through the action `request` param
     * @return {Object}         The Request URL
     */

  }, {
    key: 'getUrl',
    value: function getUrl(_ref5) {
      var wpRequest = _ref5.wpRequest;

      return wpRequest._renderURI();
    }

    /**
     * Get aggregator for URL
     *
     * Infers the aggregator identifier of a given URL to which all resulting resources are going to
     * be associated with. An aggregator is a set containing resources indexed by its ids and by the
     * its custom indexers.
     *
     * @param  {String}      url          URL from which the aggregator will be infered
     * @return {String|null} aggregatorID String to which all URL direct resources will be associated
     *                                    with or null, if resources musn't be indexed
     */

  }, {
    key: 'getAggregator',
    value: function getAggregator(url) {
      var uri = url.replace(this.api._options.endpoint, '').replace(/\?.*$/, '');
      var namespace = (0, _findKey2.default)(this.api._ns, function (factory, ns) {
        return uri.indexOf(ns) === 0;
      });

      // No unregistered route/namespace will going to be indexed by default.
      // FIXME: does this works with `customRoutes`?
      if (!namespace) return null;

      // Skips namespace, takes fragments and preserves only static parts
      uri = uri.replace(namespace + '/', '');
      var fragments = uri.split('/');
      var resource = (0, _camelCase2.default)(fragments[0]);

      if (!this.api[resource]) {
        throw new Error('Unregistered route, expected \'' + resource + '\' to resolve to \'' + url + '\'');
      }

      var query = this.api[resource]();
      var aggregator = resource;

      var _loop = function _loop(piece) {
        var id = (0, _find2.default)(query._levels[piece], function (cmp) {
          return cmp.validate(fragments[piece]);
        });

        if (!id) {
          return {
            v: false
          };
        }

        if (!id.component.match(WPAPIAdapter.namedGroupRegex)) {
          aggregator += (0, _capitalize2.default)(id.component);
        }
      };

      for (var piece = 1; piece < fragments.length; piece++) {
        var _ret = _loop(piece);

        if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
      }

      // usersMe should be aggregated together with users
      if (aggregator === 'usersMe') return 'users';

      return aggregator;
    }

    /**
     * Define a time to live for request
     *
     * It allows adapter define the ttl per request basis
     *
     * @param  {Object} request Request provided by consumer through action `request` param
     * @return {Number}         The time to live of request's resources
     */

  }, {
    key: 'getTTL',
    value: function getTTL(request) {
      return request.ttl || this.defaultTTL;
    }

    /**
     * Retrieves request operation
     *
     * @param  {Object} request Request provided by consumer through action `request` param
     * @return {('get'|'create'|'update'|'delete')} The request operation
     */

  }, {
    key: 'getOperation',
    value: function getOperation(request) {
      return request.operation;
    }

    /**
     * Builds the Request object
     *
     * Deals with consumer input through an action and produces the request to attend be later
     * fetched. The request must carry at least operation (get|create|update\delete) and required data
     * in order to call API later at `callAPI`.
     *
     * @param  {Object} payload                  The action payload
     * @param  {Object} payload.request          The lib consumer input for calling the api
     * @param  {Object} payload.additionalParams additional params in order to make request, generally
     *                                           meta data such as method or header to be handled by
     *                                           `callAPI`
     * @return {Object}                          The Request Object
     */

  }, {
    key: 'buildRequest',
    value: function buildRequest(_ref6) {
      var requestBuilder = _ref6.request;
      var additionalParams = _ref6.additionalParams;

      var wpRequest = requestBuilder(this.api);
      var _additionalParams$ope = additionalParams.operation;
      var operation = _additionalParams$ope === undefined ? 'get' : _additionalParams$ope;
      var ttl = additionalParams.ttl;

      var body = _objectWithoutProperties(additionalParams, ['operation', 'ttl']);

      return { wpRequest: wpRequest, operation: operation, body: body, ttl: ttl };
    }

    /**
     * Sends API request
     *
     * It is expected to return a promise which resolves to a data consumable by both `getBody` and
     * `getPagination`. In case of rejection, a reason is expected under message property from Error.
     *
     * @param  {Object}  request Provided by consumer through the action `request` param
     * @return {Promise}         The future result of the operation
     */

  }, {
    key: 'sendRequest',
    value: function sendRequest(_ref7) {
      var wpRequest = _ref7.wpRequest;
      var operation = _ref7.operation;

      // Embeds any directly embeddable resource linked to current resource(s)
      return wpRequest.embed()[operation](wpRequest._body);
    }
  }]);

  return WPAPIAdapter;
}();

WPAPIAdapter.namedGroupRegex = new RegExp([
// Capture group start
'\\(\\?',
// Capture group name begins either `P<`, `<` or `'`
'(?:P<|<|\')',
// Everything up to the next `>`` or `'` (depending) will be the capture group name
'([^>\']+)',
// Capture group end
'[>\']',
// Get everything up to the end of the capture group: this is the RegExp used
// when matching URLs to this route, which we can use for validation purposes.
'([^\\)]*)',
// Capture group end
'\\)'].join(''));
exports.default = WPAPIAdapter;