'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.selectQuery = exports.selectRequest = exports.selectRequestRaw = exports.withDenormalize = exports.denormalizeTree = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _isFunction = require('lodash/isFunction');

var _isFunction2 = _interopRequireDefault(_isFunction);

var _reselect = require('reselect');

var _isString = require('lodash/isString');

var _isString2 = _interopRequireDefault(_isString);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _requestStatus = require('./constants/requestStatus');

var _helpers = require('./helpers');

var _symbols = require('./symbols');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var requestsByQuery = function requestsByQuery(state) {
  return state.wp.getIn(['requestsByQuery']);
};
var localResources = function localResources(state) {
  return state.wp.getIn(['resources']);
};

var denormalizeTree = exports.denormalizeTree = function denormalizeTree(resources, id) {
  var memoized = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  /* eslint-disable no-param-reassign, no-underscore-dangle */
  if (memoized[id] && memoized[id][_symbols.lastCacheUpdate] === resources.get(id)[_symbols.lastCacheUpdate]) {
    return memoized[id];
  }

  var resource = resources.get(id);
  if (!resource) {
    return null;
  }

  memoized[id] = _extends(_defineProperty({}, _symbols.id, id), resource, (0, _helpers.mapDeep)(resource._embedded || {}, function (embeddedId) {
    return denormalizeTree(resources, embeddedId, memoized);
  }));

  return memoized[id];
};

var withDenormalize = exports.withDenormalize = function withDenormalize(thunk) {
  var previousValue = void 0;
  var previousThunkValue = void 0;
  var memo = {};

  return function (state) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    var nextValue = thunk.apply(undefined, [state].concat(args));
    if (nextValue === previousValue) {
      return previousThunkValue;
    }

    if (!(0, _isFunction2.default)(nextValue)) {
      previousValue = nextValue;
      previousThunkValue = nextValue;
      return nextValue;
    }

    var resources = localResources(state);
    previousValue = nextValue;
    previousThunkValue = nextValue(function (id) {
      return denormalizeTree(resources, id, memo);
    });
    return previousThunkValue;
  };
};

/**
 * Adds pagination when available
 */
function formatRequestRaw(request, cache) {
  if (!request) return { status: _requestStatus.pending, data: false, error: false };

  var requestFormatted = request;
  var cacheID = request.get('cacheID');
  var page = request.get('page');

  if (cacheID) {
    requestFormatted = request.merge(cache.getIn([cacheID, page])).merge(cache.getIn([cacheID, 'pagination']));
  }

  requestFormatted = requestFormatted.toJSON();

  if (!requestFormatted.data) {
    return _extends({ data: false, error: false }, requestFormatted);
  }

  return requestFormatted;
}

/**
 * Creates a selector without denormalization to resolve to a Request
 *
 * Creates a selector to resolve to a Request based on requestID, which might be either its name
 * or a cacheID and page. It doesn't do any denormalization, so data contains local ids.
 *
 * @param   {String|Object} requestID If string, the name of the request, if Object, the cacheID
 *                                    and the page for that request.
 *
 * @returns {Function}                Selector to pick from state the request.
 */
var selectRequestRaw = exports.selectRequestRaw = function selectRequestRaw(requestID) {
  if (!requestID) {
    throw new Error('\'requestID\' must either be the request name or an object with its cacheID and page');
  }

  if ((0, _isString2.default)(requestID)) {
    return (0, _reselect.createSelector)(function (state) {
      return state.wp.getIn(['requestsByName', requestID]);
    }, requestsByQuery, formatRequestRaw);
  }

  if (requestID.name) {
    return (0, _reselect.createSelector)(function (state) {
      return state.wp.getIn(['requestsByName', requestID.name]).merge(requestID);
    }, requestsByQuery, formatRequestRaw);
  }

  if (!requestID.cacheID) {
    throw new Error('\'cacheId\' or \'name\' must be provided to select a request');
  }

  var request = new _immutable2.default.Map(_extends({
    page: 1,
    data: false
  }, requestID));

  return function () {
    return formatRequestRaw(request, requestsByQuery.apply(undefined, arguments));
  };
};

/**
 * creates selector for a request
 *
 * Creates a selector by its name or by its cacheID and page.
 *
 * @param   {String|Object} requestID If string, the name of the request, if Object, the cacheID
 *                                    and the page for that request.
 * @returns {Function}                Selector which accepts the app state and returns the request
 */
var selectRequest = exports.selectRequest = function selectRequest(requestID) {
  return withDenormalize((0, _reselect.createSelector)(selectRequestRaw(requestID), function (requestRaw) {
    return function (denormalize) {
      if (!requestRaw.data) {
        return requestRaw;
      }

      return _extends({}, requestRaw, {
        data: requestRaw.data.map(denormalize)
      });
    };
  }));
};

/**
 * @deprecated use selectRequest instead
 */
var selectQuery = exports.selectQuery = function selectQuery(name) {
  // eslint-disable-next-line no-console
  console.warn('Deprecation warning: `selectQuery` was renamed to `selectRequest`');
  return selectRequest(name);
};